package ex5;

import java.util.Vector;
import javax.realtime.WaitFreeWriteQueue;

/**
 * The data communications class used to buffer the data sourced by the real-time
 * system activity, and consumed by non-realtime system activity.
 */
public class DataBuffer {
    // the main data store accessed by non-real-time consumer threads
    final Vector<Double> dataset;
    
    // the internal buffer used by no-heap entities to store their data
    final WaitFreeWriteQueue queue;

    public DataBuffer() {
        dataset = new Vector<Double>(10000);
        // pre-populate to simulate an initial, partial data set
        for (int i = 0; i < 1000; i++) {
            dataset.add(new Double(Math.random()));
        }
        process();
        queue = new WaitFreeWriteQueue(10000);
    }

    /** Add the given value to the data buffer
     *
     * @param d the data to add
     */
    public void add(Double d) {
        // Dealing with buffer overflow requires careful
        // consideration of producer/consumer rates in
        // general. For simplicity we use the force()
        // method, which can't fail due to a full buffer
        queue.force(d);
    }

    /**
     * Remove and return the head element in the data set
     * @return the head element, or null if the data set is empty
     */
    public Double take() {
        // still synchronize on dataset to ensure only one take() can
        // execute at a time
        synchronized (dataset) {
            // copy everything currently in the buffer into the main
            // dataset and then process it
            while (!queue.isEmpty()) {
                try {
                    dataset.add((Double) queue.read());
                } catch (InterruptedException ex) {
                    throw new InternalError("shouldn't happen");
                }
            }
            if (dataset.size() > 0) {
                process(); // simulate data processing activity
                return dataset.remove(0);
            }
        }
        return null;
    }

    /**
     * Simulates a data-processing operation - in this case a simple and
     * slow bubble-sort
     */
    private void process() {
        boolean sorted;
        do {
            sorted = true;
            for (int i = 0; i < dataset.size() - 1; i++) {
                if (dataset.get(i).compareTo(dataset.get(i + 1)) < 0) {
                    Double d1 = dataset.get(i);
                    Double d2 = dataset.get(i + 1);
                    dataset.set(i, d2);
                    dataset.set(i + 1, d1);
                    sorted = false;
                }
            }
        } while (!sorted);
    }
}
